// PRAL.cpp : This file contains the 'main' function. Program execution begins and ends there.
//

#include <iostream>
#include <functional>
#include <numbers>
#include <vector>
#include "trim_electronic_loss_coff.h"
/*
inline bool compare_VectorXd(Eigen::VectorXd v1, Eigen::VectorXd v2) {
    for (int i = 0; i < v1.rows(); i++) {
        if (v1(i) > v2(i)) {
            return true;
            break;
        }
    }
    return false;
}
Eigen::VectorXd runge_kutta_fehlberg(std::function<Eigen::VectorXd(double, Eigen::VectorXd)>f,double xinit, Eigen::VectorXd yinit,Eigen::VectorXd error_expt) {
    double h = 1;
    double x=xinit;
    Eigen::VectorXd y=yinit;
    Eigen::VectorXd k1,k2,k3,k4,k5,k6,y1,error;

    do{
    k1=h*f(x,y);
    k2=h*f(x+2.0/9*h,y+2.0/9*k1);
    k3=h*f(x+1.0/3*h,y+1.0/12*k1+1.0/4*k2);
    k4=h*f(x+3.0/4*h,y+69.0/128*k1-243.0/128*k2+135.0/64*k3);
    k5=h*f(x+h,y-17.0/12*k1+27.0/4*k2-27.0/5*k3+16.0/15*k4);
    k6=h*f(x+5.0/6*h,y+65.0/432*k1-5.0/16*k2+13.0/16*k3+4.0/27*k4+5.0/144*k5);
    y1=y+47.0/150*k1+12.0/25*k3+32.0/225.0*k4+1.0/30.0*k5+6.0/25.0*k6;
    error=(-1.0/150*k1+3.0/100*k3-16.0/75*k4-1.0/20*k5+6.0/25*k6).cwiseAbs ();
    } while (compare_VectorXd(error,y1));
    for()
}
*/

void pral(double EMAX, unsigned Z1, unsigned Z2, double M1, double M2,double density) {
    double number_density = density * 0.6023 / M2;
    double arho = density * 0.6023 / M2;
    double mu = M2 / M1;
    double gamma = 4.0 * mu / pow(1.0 + mu, 2);
    double a = 0.4685 / (pow(Z1, 0.23) + pow(Z2, 0.23));
    double epsilon_coff = a * mu / ((1 + mu) * Z1 * Z2 * 14.4);
    double epsilon_nuclear_power_coff = std::numbers::pi * pow(a, 2) * gamma * number_density / epsilon_coff;
    double xi_coff = 1.0 / (number_density * std::numbers::pi * pow(a, 2) * gamma);

    double E = 10000.0;//单位eV
    double epsilon = E * epsilon_coff;

    auto nuclear_stopping_epsilon_equation = [](double epsilon) {return 0.5 * log(1 + 1.1383 * epsilon) / (epsilon + 0.01321 * pow(epsilon, 0.21226) + 0.19594 * sqrt(epsilon)); };

    double nuclear_stopping=epsilon_nuclear_power_coff* nuclear_stopping_epsilon_equation(epsilon);

    auto nuclear_straggling_epsilon_equation = [](double epsilon) {return 1.0 / (4 + 0.197 * pow(epsilon, -0.16991) + 6.68400 * pow(epsilon, -1.0494)); };
    double nuclear_straggling = nuclear_straggling_epsilon_equation(epsilon) * gamma / epsilon_coff * epsilon_nuclear_power_coff;

    double electronic_stopping = calculate_elec_loss_coff_trim85(Z1, M1, Z2, E/1000) * number_density;

    double total_stopping = nuclear_stopping + electronic_stopping;
    //初始值设置
    double nu = mu / (1.2 * (1.0 + electronic_stopping / nuclear_stopping));
    double beta = 1.0 / (1.0 - nuclear_straggling / (gamma * E * nuclear_stopping));

    double x_bar = 1342;// 2.0 * E / (mu * nuclear_stopping * (1.0 + 1.0 / nu));
    //double x_elec = E / (0.4 * electronic_stopping);
    //double x_total = E / (0.4 * total_stopping);
    double xi =  2308469;// (1.0 + nu)* pow(x_bar, 2);
    
    double sigma_z2 = pow(488,2);
    double sigma_x2;// = xi - sigma_z2 - pow(x_bar, 2);

    double E_coff = pow(10.0, 0.1 / 100);
    double sigma_x;// = sqrt(sigma_x2);
    double sigma_z;// = sqrt(sigma_z2);



    
    while (1) {
        double E0=E;
        E *= E_coff;
        double dE = E-E0;
        epsilon = E * epsilon_coff;

        nuclear_stopping = epsilon_nuclear_power_coff * nuclear_stopping_epsilon_equation(epsilon);
        nuclear_straggling = nuclear_straggling_epsilon_equation(epsilon) * gamma / epsilon_coff * epsilon_nuclear_power_coff;
        electronic_stopping = calculate_elec_loss_coff_trim85(Z1, M1, Z2, E/1000) * number_density;
        total_stopping = nuclear_stopping + electronic_stopping;

        double dx_bar = (1 - (mu * nuclear_stopping / (2.0 * E) + mu * nuclear_straggling / (4 * pow(E, 2))) * x_bar) / (total_stopping - mu * nuclear_straggling / (2.0 * E));
        double dxi = 2.0 * x_bar / total_stopping;
        double dsigma_z2;
        
        double dsigma_z2_val = total_stopping - 2.0 * mu * nuclear_straggling / E;
        if (dsigma_z2_val/total_stopping>=0.01) {
            dsigma_z2 = (-2.0 * mu * nuclear_straggling * x_bar / E / nuclear_stopping + (mu * nuclear_stopping / E - pow(1.0 - mu, 2) * nuclear_straggling / (4.0 * pow(E, 2))) * (xi - 2.0 * sigma_z2)) / dsigma_z2_val;
            sigma_z2 += dsigma_z2 * dE;
        }
        else {
            sigma_z2 = 0.5 * xi - mu * nuclear_straggling * x_bar / (total_stopping * (mu * nuclear_stopping - pow(1.0 - mu, 2) * nuclear_straggling / (4.0 * E)));
        }
        x_bar += dx_bar*dE;
        xi += dxi*dE;

        sigma_x2 = xi - sigma_z2 - x_bar * x_bar;
        sigma_x = sqrt(sigma_x2);
        sigma_z = sqrt(sigma_z2);
        if (E < EMAX) {
            std::cout << E << " " << x_bar <<" "<< sigma_z << std::endl;
        }
        else {
            break;
        }
    }
    //std::cout << E << " " << x_bar << " " << sigma_z << std::endl;
}

void pral_revised_1st_order(double EMAX, unsigned Z1, unsigned Z2, double M1, double M2, double density, std::vector<std::vector<double>>& result_out) {
    double number_density = density * 0.6023 / M2;
    double mu = M2 / M1;
    double gamma = 4.0 * mu / pow(1.0 + mu, 2);
    double a = 0.4685 / (pow(Z1, 0.23) + pow(Z2, 0.23));
    double epsilon_coff = a * mu / ((1 + mu) * Z1 * Z2 * 14.4);
    double epsilon_nuclear_power_coff = std::numbers::pi * pow(a, 2) * gamma * number_density / epsilon_coff;
    double xi_coff = 1.0 / (number_density * std::numbers::pi * pow(a, 2) * gamma);

    double E = 10.0;//单位eV
    double epsilon = E * epsilon_coff;

    auto nuclear_stopping_epsilon_equation = [](double epsilon) {return 0.5 * log(1 + 1.1383 * epsilon) / (epsilon + 0.01321 * pow(epsilon, 0.21226) + 0.19594 * sqrt(epsilon)); };

    double nuclear_stopping = epsilon_nuclear_power_coff * nuclear_stopping_epsilon_equation(epsilon);

    auto nuclear_straggling_epsilon_equation = [](double epsilon) {return 1.0 / (4 + 0.197 * pow(epsilon, -0.16991) + 6.68400 * pow(epsilon, -1.0494)); };
    double nuclear_straggling = nuclear_straggling_epsilon_equation(epsilon) * gamma / epsilon_coff * epsilon_nuclear_power_coff;

    double electronic_stopping = calculate_elec_loss_coff_trim85(Z1, M1, Z2, E / 1000) * number_density;

    double total_stopping = nuclear_stopping + electronic_stopping;
    //初始值设置
    double nu = mu / (1.2 * (1.0 + electronic_stopping / nuclear_stopping));
    double beta = 1.0 / (1.0 - nuclear_straggling / (gamma * E * nuclear_stopping));

    double x_bar = 2.0 * E / (mu * nuclear_stopping * (1.0 + 1.0 / nu));
    //double x_elec = E / (0.4 * electronic_stopping);
    //double x_total = E / (0.4 * total_stopping);
    double xi = (1.0 + nu) * pow(x_bar, 2);

    double sigma_z2 = xi / (2.0 + beta / nu);
    double x_bar2 = xi - sigma_z2;

    double E_coff = pow(10.0, 0.1 / 100);
    double sigma_x;// = sqrt(sigma_x2);
    double sigma_z;// = sqrt(sigma_z2);




    while (1) {
        double E0 = E;
        E *= E_coff;
        double dE = E - E0;
        epsilon = E * epsilon_coff;

        nuclear_stopping = epsilon_nuclear_power_coff * nuclear_stopping_epsilon_equation(epsilon);
        nuclear_straggling = nuclear_straggling_epsilon_equation(epsilon) * gamma / epsilon_coff * epsilon_nuclear_power_coff;
        electronic_stopping = calculate_elec_loss_coff_trim85(Z1, M1, Z2, E / 1000) * number_density;
        total_stopping = nuclear_stopping + electronic_stopping;

        double dx_bar = (1 - mu * nuclear_stopping * x_bar / (2.0 * E)) / total_stopping;
        double dx_bar2 = (2.0 * x_bar - mu * nuclear_stopping / E * (x_bar2 - sigma_z2)) / total_stopping;
        double dsigma_z2 = 0.5 * mu * nuclear_stopping / E * (x_bar2 - sigma_z2) / total_stopping;

        x_bar += dx_bar * dE;
        x_bar2 += dx_bar2 * dE;
        sigma_z2 += dsigma_z2 * dE;
        sigma_z = sqrt(sigma_z2);

        if (E < EMAX) {
            result_out.push_back({ E,x_bar,sqrt(x_bar2 - pow(x_bar, 2)),sigma_z });
        }
        else {
            result_out.push_back({ E,x_bar,sqrt(x_bar2 - pow(x_bar, 2)),sigma_z });
            break;
        }
    }
}
double line_interpolate(double x, const std::vector<std::vector<double>>& vec) {
    unsigned i;
    for (i = 0; i < vec.size() - 1; i++) {
        if (x < vec[i + 1][0]) {
            break;
        }
    }
    return vec[i][1] + (vec[i + 1][1] - vec[i][1]) / (vec[i + 1][0] - vec[i][0]) * (x - vec[i][0]);
}
int main()
{
    std::vector<std::vector<double>> result_out;
    pral_revised_1st_order(10000*1000, 1, 14, 1.0078, 27.977, 2.3212,result_out);
    std::vector<double> E = { 10,11,12,13,14,15,16,17,18,20,22.5000000000000,25,27.5000000000000,30,32.5000000000000,35,37.5000000000000,40,45,50,55,60,65,70,80,90,100,110,120,130,140,150,160,170,180,200,225,250,275,300,325,350,375,400,450,500,550,600,650,700,800,900,1000,1100,1200,1300,1400,1500,1600,1700,1800,2000,2250,2500,2750,3000,3250,3500,3750,4000,4500,5000,5500,6000,6500,7000,8000,9000,10000 };
    for (auto e : E) {
        std::cout << e<<" "<< line_interpolate(e*1000,result_out) << std::endl;
    }
    
}

// Run program: Ctrl + F5 or Debug > Start Without Debugging menu
// Debug program: F5 or Debug > Start Debugging menu

// Tips for Getting Started: 
//   1. Use the Solution Explorer window to add/manage files
//   2. Use the Team Explorer window to connect to source control
//   3. Use the Output window to see build output and other messages
//   4. Use the Error List window to view errors
//   5. Go to Project > Add New Item to create new code files, or Project > Add Existing Item to add existing code files to the project
//   6. In the future, to open this project again, go to File > Open > Project and select the .sln file
